@@ -9,6 +9,7 @@ public:
 	virtual void Hv(double *s, double *Hs) = 0 ;
 
 	virtual int get_nr_variable(void) = 0 ;
+	virtual void get_diagH(double *M) = 0 ;
 	virtual ~function(void){}
 };
 
@@ -22,7 +23,7 @@ public:
 	void set_print_string(void (*i_print) (const char *buf));
 
 private:
-	int trcg(double delta, double *g, double *s, double *r, bool *reach_boundary);
+	int trpcg(double delta, double *g, double *M, double *s, double *r, bool *reach_boundary);
 	double norm_inf(int n, double *x);
 
 	double eps;
